unit VirtList;
{
  John Cardinal
  JCardinal@ElmSquare.com

  VirtList implements 2 "virtual listbox" components, TCustomVirtualListBox and
  TVirtualListBox, which are extensions of the owner-drawn listbox.  These use
  the Windows 95 LBS_NODATA style to specify a no-data listbox. A no-data listbox
  is very similar to an owner-drawn listbox, except that Windows does not manage
  the data for the list: the listbox contains no item data.

  Using the class is simple: you drop the component on the form, specify the
  properties, and implement an OnDrawItem (TDrawItemEvent) procedure.  This
  procedure is passed the line number to be drawn.  It's up to you to decide
  what to draw.  Typically, the line number will index a data structure
  maintained by your application.

  A no-data listbox does not have an Items property, because, well... that's the
  whole idea, isn't it? There is no Sorted property, either, because the no-data
  listbox doesn't know how to move items around.

  The virtual listbox has one important new property: Count.  The value of Count
  determines how many items are displayed. It replaces Items.Count, which isn't
  accessable because the Items property isn't public/published.

  There is also a ScrollBars property, which is missing from the standard
  TListBox component. (Why?)

  Version History:
  1.0   First uploaded version
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TCustomVirtualListBox = class(TCustomListBox)
  private
    { Private declarations }
  protected
    { protected declarations }
    FCount: Smallint;
    FSaved: Boolean;
    FSaveIndex: Smallint;
    FSaveTop: Smallint;
    FScrollBars: TScrollStyle;
    procedure CreateHandle; override;
    procedure DestroyWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure SetCount(value: Smallint);
    procedure SetScrollBars(Value: TScrollStyle);

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure CreateParams(var Params: TCreateParams); override;
    property Count: Smallint read FCount write SetCount;
    property ScrollBars: TScrollStyle read FScrollBars write SetScrollBars default ssBoth;
  end;

  TVirtualListBox = class(TCustomVirtualListBox)
  published
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    //property Items;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ScrollBars;
    property ShowHint;
    //property Sorted;
    //property Style;
    property TabOrder;
    property TabStop;
    property TabWidth;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    //property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  procedure Register;

implementation

{ TVirtualListBox }

constructor TCustomVirtualListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FScrollBars := ssBoth;
  FSaved := False;
end;

procedure TCustomVirtualListBox.CreateHandle;
begin
  inherited CreateHandle;
  Perform(LB_SETCOUNT, FCount, 0);
  if FSaved then begin
    ItemIndex := FSaveIndex;
    TopIndex := FSaveTop;
  end;
  FSaved := False;
end;

procedure TCustomVirtualListBox.CreateParams(var Params: TCreateParams);
const
  ScrollBar: array[TScrollStyle] of LongInt = (0, WS_HSCROLL, WS_VSCROLL,
    WS_HSCROLL or WS_VSCROLL);
begin
  inherited CreateParams(Params);
  with Params do begin
    Style := Style and (not (LBS_SORT or LBS_HASSTRINGS or LBS_OWNERDRAWVARIABLE
                    or WS_HSCROLL or WS_VSCROLL));
    Style := Style or LBS_OWNERDRAWFIXED or LBS_NODATA or ScrollBar[FScrollBars];
  end;
end;

procedure TCustomVirtualListBox.DestroyWnd;
begin
  if FCount > 0 then begin
    FSaveIndex := ItemIndex;
    FSaveTop := TopIndex;
    Items.Clear;
    FSaved := True;
  end;
  inherited DestroyWnd;
end;

procedure TCustomVirtualListBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
begin
  if Assigned(OnDrawItem) then
    OnDrawItem(Self, Index, Rect, State)
  else
    Canvas.FillRect(Rect);
end;

procedure TCustomVirtualListBox.SetCount(value: Smallint);
begin
  FCount := value;
  Perform(LB_SETCOUNT, FCount, 0);
end;

procedure TCustomVirtualListBox.SetScrollBars(Value: TScrollStyle);
begin
  if FScrollBars <> Value then begin
    FScrollBars := Value;
    RecreateWnd;
  end;
end;

procedure Register;
begin
  RegisterComponents('Elm', [TVirtualListBox]);
end;

end.
